"""
A simple animation system for pygame sprites.
designed for my simple games. 
i am open for feedback :)
"""

from .animation import Animation
from .animation_player import AnimationPlayer

__version__ = "1.0.0"
__author__ = "de-dust-enjoyer"

__all__ = ["Animation", "AnimationPlayer"]